h1, h2, h3, h4 {
    color: rgb(80, 80, 80); /* We want the headers dark gray */
}

body {
    color: rgb(80, 80, 80); /* Dark gray text */
    /* background: linear-gradient(90deg, rgba(240,240,240,1) 0%, rgba(240,240,240,1) 50%, rgba(150,150,195,1) 100%);*/
    /* background: linear-gradient(90deg, rgba(200,200,220,1) 0%, rgba(150,150,195,1) 100%); */
    background: linear-gradient(90deg, rgba(200,200,220,1) 0%, rgba(170,170,210,1) 100%);
}

/* Custom CSS to style bold text */
b, strong {
    color: rgb(40, 40, 40);
}

/* Reduce spacing around lists */
p {
    margin-bottom: 0 !important;
}
ul, ol {
    margin-top: 0 !important;
    margin-bottom: 0.25em !important;
}

/* AgGrid custom CSS */

/* Adjust header background color */
.ag-header {
    --ag-header-background-color: rgba(150, 150, 195);
}

/* Adjust cell background */
.ag-cell {
    background-color: rgb(240, 240, 240);
}

/* Alternate row colors */
.ag-row:nth-child(even) .ag-cell {
    background-color: rgb(230, 230, 230);
}

/* Selection color for the entire row */
.ag-row.ag-row-selected .ag-cell {
    background-color: rgba(170, 170, 205, 1.0);
}

/* There's a one pixel border around the grid that we want to remove */
.ag-root-wrapper {
    border: none !important; /* Force removal with !important */
}


/* Box shadow and rounded corners for all AgGrid themes */
[class*="ag-theme-"] {
    box-shadow: 2px 2px 6px 5px rgba(0, 0, 0, 0.2); /* Drop shadow */
    border-radius: 12px; /* Rounded corners */
    border: 0.5px solid rgba(0, 0, 205, 0.5);
    margin: 0;
    padding: 0;
}

/* Apply styling to Workbench containers */
.workbench-container {
    background-color: rgba(240, 240, 240, 0.5);
    box-shadow: 2px 2px 6px 5px rgba(0, 0, 0, 0.2);
    border-radius: 12px; /* Rounded corners */
}

/* Apply styling to Workbench Offsets */
.workbench-offset {
    background-color: rgba(240, 240, 240, 0.25);
    box-shadow: 1px 1px 3px 2px rgba(0, 0, 0, 0.2);
    border-radius: 12px; /* Rounded corners */
}

/* Apply styling to Workbench Highlight */
.workbench-highlight {
    background-color: rgba(240, 240, 240, 0.5);
    box-shadow: 1px 1px 3px 2px rgba(0, 0, 0, 0.2);
    border-radius: 12px; /* Rounded corners */
}

/* Apply styling to custom tooltips */
.custom-tooltip {
    background-color: rgba(240, 240, 240, 1.0);
    box-shadow: 2px 2px 6px 5px rgba(0, 0, 0, 0.25);
    border-radius: 25px; /* Rounded corners */
    overflow: hidden; /* Ensure contents fit inside the rounded corners */
    border: 2px solid rgba(128, 128, 128, 1); /* 1-pixel grey */
}

/* Some of the HTML/Markdown will use color hints like 'green-text' or 'blue-text' */
.green-text {
    color: rgb(40, 100, 40);
}

.blue-text {
    color: rgb(70, 70, 160);
}

.pink-text {
    color: rgb(200, 100, 200);
}

.red-text {
    color: rgb(200, 60, 100);
}

.orange-text {
    color: rgb(160, 80, 0);
}

.alert {
    color: rgb(200, 60, 100);
}

.warning {
    color: rgb(160, 80, 0);
}

.good {
    color: rgb(40, 100, 40);
}


/* Table styling */
table {
    width: 100%;
}

th {
    padding: 10px;
    border: 1px solid rgb(120, 120, 120);
    background-color: rgba(150, 150, 195);
    font-weight: bold;
    text-align: center !important;
}

td {
    padding: 5px;
    border: 0.5px solid rgb(120, 120, 120);
    background-color: rgba(240, 240, 240, 0.5);
    text-align: center !important;
}
