.dash-graph.scatter-plot-light {
    background-color: #ffffff; /* White background */
    color: #000000; /* Black text for titles and labels */
}

.dash-graph.scatter-plot-dark {
    background-color: #333333; /* Dark gray background */
    color: #ffffff; /* White text for titles and labels */
}

/*.ag-theme-custom {*/
/*    --ag-header-height: 30px !important;*/
/*}*/

.ag-theme-custom .ag-header {
    background: linear-gradient(rgba(255, 255, 255, 0.5), rgb(115, 190, 115));
}
.ag-theme-custom-dark .ag-header {
    background: linear-gradient(rgb(60, 100, 60), rgba(0, 0, 0, 0.5));
}


