h1, h2, h3, h4 {
    color: rgb(220, 220, 220); /* We want the headers off white */
}
body {
    color: rgb(180, 180, 180); /* We want the text dim white */
}

/* Custom CSS to style bold text */
b, strong {
    color: rgb(120, 120, 240); /* Replace with your desired RGB color */
}

/* Custom CSS to style italic text */
i, em {
    color: rgb(160, 120, 240); /* Replace with your desired RGB color */
}

/* Default link color */
a {
    color: rgb(120, 120, 240); /* Change to your preferred color */
}

/* Reduce spacing around lists */
p {
    margin-bottom: 0 !important;
}
ul, ol {
    margin-top: 0 !important;
    margin-bottom: 0.25em !important;
}

/* Hover effect */
a:hover {
    color: rgb(160, 160, 255); /* Change hover color */
}

/* AgGrid custom CSS */

/* There's a one pixel border around the grid that we want to remove */
.ag-root-wrapper {
    border: none !important; /* Force removal with !important */
}


/* Box shadow and rounded corners for all AgGrid themes */
[class*="ag-theme-"] {
    box-shadow: 2px 2px 6px 5px rgba(0, 0, 0, 0.25);
    border-radius: 12px; /* Rounded corners */
    border: 0.5px solid rgba(0, 0, 0, 0.5);
    margin: 0;
    padding: 0;
}

/* Apply styling to Workbench containers */
.workbench-container {
    box-shadow: 2px 2px 6px 5px rgba(0, 0, 0, 0.25);
    border-radius: 12px; /* Rounded corners */
    background-color: rgb(40, 40, 40); /* Dark background */
}

/* Apply styling to Workbench Offsets */
.workbench-offset {
    box-shadow: 1px 1px 3px 2px rgba(0, 0, 0, 0.25);
    border-radius: 12px; /* Rounded corners */
}

/* Apply styling to custom tooltips */
.custom-tooltip {
    background-color: rgba(40, 40, 40, 1.0);
    box-shadow: 2px 2px 6px 5px rgba(0, 0, 0, 0.25);
    border-radius: 25px; /* Rounded corners */
    overflow: hidden; /* Ensure contents fit inside the rounded corners */
    border: 2px solid rgba(128, 128, 128, 1); /* 1-pixel grey */
}

/* Some of the HTML/Markdown will use color hints like 'green-text' or 'blue-text' */
.green-text {
    color: rgb(140, 255, 140);
}

.blue-text {
    color: rgb(140, 140, 255);
}

.pink-text {
    color: rgb(240, 140, 240);
}

.red-text {
    color: rgb(255, 140, 140);
}

.orange-text {
    color: rgb(255, 195, 140);
}

.purple-text {
    color: rgb(160, 120, 240);
}

.alert {
    color: rgb(255, 100, 140);
}

.warning {
    color: rgb(255, 165, 110);
}

.good {
    color: rgb(100, 255, 100);
}

/* Table styling */
table {
    width: 100%;
}

th {
    padding: 10px;
    border: 1px solid #444;
    background-color: rgb(40, 40, 40);
    color: rgb(160, 120, 240);
    font-weight: bold;
    text-align: center !important;
}

td {
    padding: 5px;
    border: 0.5px solid #444;
    text-align: center !important;
}